@@ -2,19 +2,15 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *                Cosmin Petra and Radu Serban @ LLNL
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2020, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * Example program for IDAS: Brusselator, parallel, GMRES, IDABBD
  * preconditioner, ASA
@@ -27,43 +23,43 @@
  * The mathematical problem solved in this example is a DAE system
  * that arises from a system of partial differential equations after
  * spatial discretization.
- * 
+ *
  * The PDE system is a two-species time-dependent PDE known as
  * Brusselator PDE and models a chemically reacting system.
  *
- *                  
- *  du/dt = eps(u  + u) + u^2 v -(B+1)u + A  
+ *
+ *  du/dt = eps(u  + u) + u^2 v -(B+1)u + A
  *               xx   yy
  *                                          domain Omega = [0,L]X[0,L]
  *  dv/dt = eps(v  + v) - u^2 v + Bu
  *               xx   yy
  *
  *  B.C. : Neumann
- *  I.C  : u(x,y,t0) = u0(x,y) =  1  - 0.5*cos(pi*y/L) 
- *         v(x,y,t0) = v0(x,y) = 3.5 - 2.5*cos(pi*x/L) 
+ *  I.C  : u(x,y,t0) = u0(x,y) =  1  - 0.5*cos(pi*y/L)
+ *         v(x,y,t0) = v0(x,y) = 3.5 - 2.5*cos(pi*x/L)
  *
  * The PDEs are discretized by central differencing on a MX by MY
- * mesh, and so the system size Neq is the product MX*MY*NUM_SPECIES. 
- * The system is actually implemented on submeshes, processor by 
- * processor, with an MXSUB by MYSUB mesh on each of NPEX * NPEY 
+ * mesh, and so the system size Neq is the product MX*MY*NUM_SPECIES.
+ * The system is actually implemented on submeshes, processor by
+ * processor, with an MXSUB by MYSUB mesh on each of NPEX * NPEY
  * processors.
  *
- * 
+ *
  * The sensitivity of the output functional
  *                            1    /
  *                   g(t) = -----  | u(x,y,t) ,
  *                          |L^2|  /
- *                               Omega 
+ *                               Omega
  * with respect to initial conditions u0 and v0 is also computed.
  * Given the perturbations du0 and dv0 in the IC, the sensitivity of
  * of g at final time tf is
  *             1    /
  *  dg(tf) = -----  | ( lambda(0,x,y) du0(x,y) + mu(0,x,y) dv0(x,y) ),
  *           |L^2|  /
- *                Omega 
+ *                Omega
  * where lambda and mu are the solutions of the adjoint PDEs:
  *
- *  dl/dt = - eps(l  + l) - (2uv - B - 1)l + (2uv - B)m  
+ *  dl/dt = - eps(l  + l) - (2uv - B - 1)l + (2uv - B)m
  *                 xx   yy
  *                                          domain Omega = [0,L]X[0,L]
  *  dm/dt = - eps(m  + m) - u^2 l + u^2 m
@@ -84,18 +80,17 @@
 #include <sunlinsol/sunlinsol_spgmr.h>
 #include <nvector/nvector_parallel.h>
 #include <sundials/sundials_types.h>
-#include <sundials/sundials_math.h>
 
 #include <mpi.h>
 
 /* Problem Constants */
-#define NUM_SPECIES 2 
+#define NUM_SPECIES 2
 #define ctL         RCONST(1.0)    /* Domain =[0,L]^2 */
 #define ctA         RCONST(1.0)
 #define ctB         RCONST(3.4)
 #define ctEps       RCONST(2.0e-3)
 
-#define PI          RCONST(3.1415926535898) /* pi */ 
+#define PI          RCONST(3.1415926535898) /* pi */
 
 #define MXSUB       21    /* Number of x mesh points per processor subgrid */
 #define MYSUB       21    /* Number of y mesh points per processor subgrid */
@@ -115,17 +110,17 @@
 #define STEPS       50
 
 #define ZERO        RCONST(0.0)
-#define HALF        RCONST(0.5) 
+#define HALF        RCONST(0.5)
 #define ONE         RCONST(1.0)
 #define TWO         RCONST(2.0)
- 
+
 /* User-defined vector accessor macro IJ_Vptr. */
 
 /*
- * IJ_Vptr is defined in order to express the underlying 3-d structure of the 
+ * IJ_Vptr is defined in order to express the underlying 3-d structure of the
  * dependent variable vector from its underlying 1-d storage (an N_Vector).
- * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to 
- * species index is = 0, x-index ix = i, and y-index jy = j.                
+ * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
+ * species index is = 0, x-index ix = i, and y-index jy = j.
  */
 
 #define IJ_Vptr(vv,i,j) (&NV_Ith_P(vv, (i)*NUM_SPECIES + (j)*NSMXSUB ))
@@ -148,8 +143,8 @@ typedef struct {
 static int res(realtype tt, N_Vector uv, N_Vector uvp,
                N_Vector rr, void *user_data);
 
-static int reslocal(sunindextype Nlocal, realtype tt, 
-                    N_Vector uv, N_Vector uvp, N_Vector res, 
+static int reslocal(sunindextype Nlocal, realtype tt,
+                    N_Vector uv, N_Vector uvp, N_Vector res,
                     void *user_data);
 
 static int rescomm(sunindextype Nlocal, realtype tt,
@@ -166,7 +161,7 @@ static void BRecvPost(MPI_Comm comm, MPI_Request request[], int thispe,
 static void BRecvWait(MPI_Request request[], int ixsub, int jysub,
                       int dsizex, realtype cext[], realtype buffer[]);
 
-static void ReactRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void ReactRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                      UserData data);
 
 /*                  ADJOINT                        */
@@ -175,13 +170,13 @@ static int resB(realtype tt, N_Vector yy, N_Vector yp,
                 N_Vector yyB, N_Vector ypB, N_Vector rrB,
                 void *user_dataB);
 
-static int resBlocal(sunindextype Nlocal, realtype tt, 
-                     N_Vector uv, N_Vector uvp, 
-                     N_Vector yyB, N_Vector ypB, 
+static int resBlocal(sunindextype Nlocal, realtype tt,
+                     N_Vector uv, N_Vector uvp,
+                     N_Vector yyB, N_Vector ypB,
                      N_Vector res, void *user_dataB);
 
 /* Prototypes for private functions */
-static void InitUserData(UserData data, int thispe, int npes, 
+static void InitUserData(UserData data, int thispe, int npes,
                          MPI_Comm comm);
 
 static void SetInitialProfiles(N_Vector uv, N_Vector uvp, N_Vector id,
@@ -191,8 +186,8 @@ static void SetInitialProfilesB(N_Vector uv, N_Vector uvp,
                                N_Vector uvB, N_Vector uvpB,
                                 N_Vector residB, UserData data);
 
-static void PrintHeader(sunindextype SystemSize, int maxl, 
-                        sunindextype mudq, sunindextype mldq, 
+static void PrintHeader(sunindextype SystemSize, int maxl,
+                        sunindextype mudq, sunindextype mldq,
                         sunindextype mukeep, sunindextype mlkeep,
                         realtype rtol, realtype atol);
 
@@ -243,13 +238,13 @@ int main(int argc, char *argv[])
 
   if (npes != NPEX*NPEY) {
     if (thispe == 0)
-      fprintf(stderr, 
-              "\nMPI_ERROR(0): npes = %d not equal to NPEX*NPEY = %d\n", 
+      fprintf(stderr,
+              "\nMPI_ERROR(0): npes = %d not equal to NPEX*NPEY = %d\n",
               npes, NPEX*NPEY);
     MPI_Finalize();
-    return(1); 
+    return(1);
   }
-  
+
   /* Set local length (local_N) and global length (SystemSize). */
   local_N = MXSUB*MYSUB*NUM_SPECIES;
   SystemSize = NEQ;
@@ -258,7 +253,7 @@ int main(int argc, char *argv[])
   data = (UserData) malloc(sizeof *data);
 
   InitUserData(data, thispe, npes, comm);
-  
+
   /* Create needed vectors, and load initial values.
      The vector resid is used temporarily only.        */
   uv  = N_VNew_Parallel(comm, local_N, SystemSize);
@@ -272,16 +267,16 @@ int main(int argc, char *argv[])
 
   id  = N_VNew_Parallel(comm, local_N, SystemSize);
   if(check_retval((void *)id, "N_VNew_Parallel", 0, thispe)) MPI_Abort(comm, 1);
-  
+
   SetInitialProfiles(uv, uvp, id, resid, data);
 
   res(ZERO, uv, uvp, resid, data);
-  
+
   /* Set remaining inputs to IDAS. */
   t0 = ZERO;
-  rtol = RTOL; 
+  rtol = RTOL;
   atol = ATOL;
-  
+
   /* Call IDACreate and IDAInit to initialize solution */
   ida_mem = IDACreate();
   if(check_retval((void *)ida_mem, "IDACreate", 0, thispe)) MPI_Abort(comm, 1);
@@ -294,7 +289,7 @@ int main(int argc, char *argv[])
 
   retval = IDAInit(ida_mem, res, t0, uv, uvp);
   if(check_retval(&retval, "IDAInit", 1, thispe)) MPI_Abort(comm, 1);
-  
+
   retval = IDASStolerances(ida_mem, rtol, atol);
   if(check_retval(&retval, "IDASStolerances", 1, thispe)) MPI_Abort(comm, 1);
 
@@ -305,14 +300,14 @@ int main(int argc, char *argv[])
 
   retval = IDASetLinearSolver(ida_mem, LS, NULL);
   if(check_retval(&retval, "IDASetLinearSolver", 1, thispe)) MPI_Abort(comm, 1);
-  
+
 
   /* Call IDABBDPrecInit to initialize the band-block-diagonal preconditioner.
      The half-bandwidths for the difference quotient evaluation are exact
      for the system Jacobian, but only a 5-diagonal band matrix is retained. */
   mudq = mldq = NSMXSUB;
   mukeep = mlkeep = 2;
-  retval = IDABBDPrecInit(ida_mem, local_N, mudq, mldq, mukeep, mlkeep, 
+  retval = IDABBDPrecInit(ida_mem, local_N, mudq, mldq, mukeep, mlkeep,
                           ZERO, reslocal, NULL);
   if(check_retval(&retval, "IDABBDPrecInit", 1, thispe)) MPI_Abort(comm, 1);
 
@@ -326,10 +321,10 @@ int main(int argc, char *argv[])
   retval = IDACalcIC(ida_mem, IDA_YA_YDP_INIT, tout);
   if(check_retval(&retval, "IDACalcIC", 1, thispe)) MPI_Abort(comm, 1);
 
-  if (thispe == 0) printf("\nStarting integration of the FORWARD problem\n\n");  
+  if (thispe == 0) printf("\nStarting integration of the FORWARD problem\n\n");
 
   /* On PE 0, print heading, basic parameters, initial values. */
-  if (thispe == 0) PrintHeader(SystemSize, maxl, 
+  if (thispe == 0) PrintHeader(SystemSize, maxl,
                                mudq, mldq, mukeep, mlkeep,
                                rtol, atol);
   /* Call IDAS in tout loop, normal mode, and print selected output. */
@@ -340,9 +335,9 @@ int main(int argc, char *argv[])
   PrintOutput(ida_mem, uv, tret, data, comm);
 
   /* Print each PE's portion of the solution in a separate file. */
-  /* PrintSol(ida_mem, uv, uvp, data, comm); */
+  PrintSol(ida_mem, uv, uvp, data, comm);
 
-  /* On PE 0, print final set of statistics. */  
+  /* On PE 0, print final set of statistics. */
   if (thispe == 0)  {
     PrintFinalStats(ida_mem);
   }
@@ -383,10 +378,10 @@ int main(int argc, char *argv[])
 
   mudq = mldq = NSMXSUB;
   mukeep = mlkeep = 2;
-  retval = IDABBDPrecInitB(ida_mem, indexB, local_N, mudq, mldq, mukeep, mlkeep, 
+  retval = IDABBDPrecInitB(ida_mem, indexB, local_N, mudq, mldq, mukeep, mlkeep,
                           ZERO, resBlocal, NULL);
   if(check_retval(&retval, "IDABBDPrecInitB", 1, thispe)) MPI_Abort(comm, 1);
-  
+
   retval = IDASolveB(ida_mem, TBEGIN, IDA_NORMAL);
   if(check_retval(&retval, "IDASolveB", 1, thispe)) MPI_Abort(comm, 1);
 
@@ -394,22 +389,22 @@ int main(int argc, char *argv[])
   if(check_retval(&retval, "IDAGetB", 1, thispe)) MPI_Abort(comm, 1);
 
   /* Print each PE's portion of solution in a separate file. */
-  /* PrintAdjSol(uvB, uvpB, data); */
+  PrintAdjSol(uvB, uvpB, data);
 
-  /* On PE 0, print final set of statistics. */  
+  /* On PE 0, print final set of statistics. */
   if (thispe == 0)  {
     PrintFinalStats(IDAGetAdjIDABmem(ida_mem, indexB));
   }
 
 
   /* Free memory. */
-  N_VDestroy_Parallel(uv);
-  N_VDestroy_Parallel(uvp);
-  N_VDestroy_Parallel(id);
-  N_VDestroy_Parallel(resid);
-  N_VDestroy_Parallel(uvB);
-  N_VDestroy_Parallel(uvpB);
-  N_VDestroy_Parallel(residB);
+  N_VDestroy(uv);
+  N_VDestroy(uvp);
+  N_VDestroy(id);
+  N_VDestroy(resid);
+  N_VDestroy(uvB);
+  N_VDestroy(uvpB);
+  N_VDestroy(residB);
 
   IDAFree(&ida_mem);
   SUNLinSolFree(LS);
@@ -429,10 +424,10 @@ int main(int argc, char *argv[])
  */
 
 /*
- * InitUserData: Load problem constants in data (of type UserData).   
+ * InitUserData: Load problem constants in data (of type UserData).
  */
 
-static void InitUserData(UserData data, int thispe, int npes, 
+static void InitUserData(UserData data, int thispe, int npes,
                          MPI_Comm comm)
 {
   data->jysub = thispe / NPEX;
@@ -482,9 +477,9 @@ static void SetInitialProfiles(N_Vector uv, N_Vector uvp, N_Vector id,
     y = (jy + jysub*mysub) * dy;
     for (ix = 0; ix < mxsub; ix++) {
       x = (ix + ixsub*mxsub) * dx;
-      uvxy = IJ_Vptr(uv,ix,jy); 
+      uvxy = IJ_Vptr(uv,ix,jy);
 
-      uvxy[0] = RCONST(1.0) - HALF*cos(PI*y/L); 
+      uvxy[0] = RCONST(1.0) - HALF*cos(PI*y/L);
       uvxy[1] = RCONST(3.5) - RCONST(2.5)*cos(PI*x/L);
     }
   }
@@ -524,7 +519,7 @@ static void SetInitialProfiles(N_Vector uv, N_Vector uvp, N_Vector id,
       uvxy1 = IJ_Vptr(uv,1,jy);
       uvxy[0] = uvxy1[0];
       uvxy[1] = uvxy1[1];
-    }    
+    }
   }
 
   if (jysub == npey-1) {
@@ -568,11 +563,11 @@ static void SetInitialProfilesB(N_Vector uv, N_Vector uvp,
   /* Loop over grid, load (lambda, mu) values. */
   for (jy = 0; jy < mysub; jy++) {
     for (ix = 0; ix < mxsub; ix++) {
-      uvBxy = IJ_Vptr(uvB,  ix,jy); 
-      uvpBxy = IJ_Vptr(uvpB,ix,jy); 
+      uvBxy = IJ_Vptr(uvB,  ix,jy);
+      uvpBxy = IJ_Vptr(uvpB,ix,jy);
+
+      uvxy = IJ_Vptr(uv,ix,jy);
 
-      uvxy = IJ_Vptr(uv,ix,jy); 
-      
       uvBxy[0] = ONE;
       uvBxy[1] = ZERO;
 
@@ -608,7 +603,7 @@ static void SetInitialProfilesB(N_Vector uv, N_Vector uvp,
       uvpBxy  = IJ_Vptr(uvpB,0,jy);
 
       uvpBxy[0] = uvpBxy[1] = ZERO;
-    }    
+    }
   }
 
   if (jysub == npey-1) {
@@ -628,8 +623,8 @@ static void SetInitialProfilesB(N_Vector uv, N_Vector uvp,
  * and table headerr
  */
 
-static void PrintHeader(sunindextype SystemSize, int maxl, 
-                        sunindextype mudq, sunindextype mldq, 
+static void PrintHeader(sunindextype SystemSize, int maxl,
+                        sunindextype mudq, sunindextype mldq,
                         sunindextype mukeep, sunindextype mlkeep,
                         realtype rtol, realtype atol)
 {
@@ -672,26 +667,26 @@ static void PrintOutput(void *ida_mem, N_Vector uv, realtype tt,
   long int nst;
   int i, kused, retval, thispe, npelast, ilast;;
 
-  thispe = data->thispe; 
+  thispe = data->thispe;
   npelast = data->npes - 1;
-  cdata = N_VGetArrayPointer_Parallel(uv);
-  
+  cdata = N_VGetArrayPointer(uv);
+
   /* Send conc. at top right mesh point from PE npes-1 to PE 0. */
   if (thispe == npelast) {
     ilast = NUM_SPECIES*MXSUB*MYSUB - 2;
     if (npelast != 0)
-      MPI_Send(&cdata[ilast], 2, PVEC_REAL_MPI_TYPE, 0, 0, comm);
+      MPI_Send(&cdata[ilast], 2, MPI_SUNREALTYPE, 0, 0, comm);
     else { clast[0] = cdata[ilast]; clast[1] = cdata[ilast+1]; }
   }
-  
+
   /* On PE 0, receive conc. at top right from PE npes - 1.
      Then print performance data and sampled solution values. */
-  
+
   if (thispe == 0) {
-    
+
     if (npelast != 0)
-      MPI_Recv(&clast[0], 2, PVEC_REAL_MPI_TYPE, npelast, 0, comm, &status);
-    
+      MPI_Recv(&clast[0], 2, MPI_SUNREALTYPE, npelast, 0, comm, &status);
+
     retval = IDAGetLastOrder(ida_mem, &kused);
     check_retval(&retval, "IDAGetLastOrder", 1, thispe);
     retval = IDAGetNumSteps(ida_mem, &nst);
@@ -700,17 +695,17 @@ static void PrintOutput(void *ida_mem, N_Vector uv, realtype tt,
     check_retval(&retval, "IDAGetLastStep", 1, thispe);
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
-    printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n", 
+    printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n",
          tt, cdata[0], clast[0], nst, kused, hused);
     for (i=1;i<NUM_SPECIES;i++)
       printf("         %12.4Le %12.4Le   |\n",cdata[i],clast[i]);
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
-    printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+    printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          tt, cdata[0], clast[0], nst, kused, hused);
     for (i=1;i<NUM_SPECIES;i++)
       printf("         %12.4e %12.4e   |\n",cdata[i],clast[i]);
 #else
-    printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+    printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          tt, cdata[0], clast[0], nst, kused, hused);
     for (i=1;i<NUM_SPECIES;i++)
       printf("         %12.4e %12.4e   |\n",cdata[i],clast[i]);
@@ -721,7 +716,7 @@ static void PrintOutput(void *ida_mem, N_Vector uv, realtype tt,
 
 }
 
-static void PrintSol(void* ida_mem, N_Vector uv, N_Vector uvp, 
+static void PrintSol(void* ida_mem, N_Vector uv, N_Vector uvp,
                      UserData data, MPI_Comm comm)
 {
   FILE* fout;
@@ -744,7 +739,7 @@ static void PrintSol(void* ida_mem, N_Vector uv, N_Vector uvp,
   mxsub = data->mxsub;
   mysub = data->mysub;
 
-  ixsub = data->ixsub;  
+  ixsub = data->ixsub;
   jysub = data->jysub;
 
   for (jy=0; jy<mysub; jy++) {
@@ -755,19 +750,19 @@ static void PrintSol(void* ida_mem, N_Vector uv, N_Vector uvp,
 
       i = ix + mxsub*ixsub;
       if(MXSUB<5 && MYSUB<5)
-        printf("PE 2D[% 2d][% 2d] -- 1D[% 2d]  subgrid[%d][%d]  uv[%d][%d] uv[[%d]]\n", 
+        printf("PE 2D[% 2d][% 2d] -- 1D[% 2d]  subgrid[%d][%d]  uv[%d][%d] uv[[%d]]\n",
                ixsub, jysub, thispe, ix, jy, i, j,
                (i)*NUM_SPECIES + (j)*NSMXSUB*npex);
-        
+
       uvxy  = IJ_Vptr(uv, ix, jy);
       /* uvxy = (&NV_Ith_P(uv, (i)*NUM_SPECIES + (j)*NSMXSUB*npex )); */
 #if defined(SUNDIALS_EXTENDED_PRECISION)
       fprintf(fout, "%Lg\n%Lg\n", uvxy[0], uvxy[1]);
 #else
       fprintf(fout, "%g\n%g\n", uvxy[0], uvxy[1]);
-#endif      
+#endif
     }
-  }    
+  }
   fclose(fout);
 }
 
@@ -799,16 +794,16 @@ static void PrintAdjSol(N_Vector uvB, N_Vector uvpB, UserData data)
       fprintf(fout, "%Lg\n%Lg\n", uvxy[0], uvxy[1]);
 #else
       fprintf(fout, "%g\n%g\n", uvxy[0], uvxy[1]);
-#endif      
+#endif
     }
-  }    
+  }
   fclose(fout);
 }
 
 
 
 /*
- * PrintFinalStats: Print final run data contained in iopt.              
+ * PrintFinalStats: Print final run data contained in iopt.
  */
 
 static void PrintFinalStats(void *ida_mem)
@@ -866,7 +861,7 @@ static void PrintFinalStats(void *ida_mem)
  *   opt == 1 means SUNDIALS function returns an integer value so check if
  *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
 static int check_retval(void *returnvalue, const char *funcname, int opt, int id)
@@ -875,25 +870,25 @@ static int check_retval(void *returnvalue, const char *funcname, int opt, int id
 
   if (opt == 0 && returnvalue == NULL) {
     /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nSUNDIALS_ERROR(%d): %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nSUNDIALS_ERROR(%d): %s() failed - returned NULL pointer\n\n",
             id, funcname);
-    return(1); 
+    return(1);
   } else if (opt == 1) {
     /* Check if retval < 0 */
     retval = (int *) returnvalue;
     if (*retval < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR(%d): %s() failed with retval = %d\n\n", 
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR(%d): %s() failed with retval = %d\n\n",
               id, funcname, *retval);
-      return(1); 
+      return(1);
     }
   } else if (opt == 2 && returnvalue == NULL) {
     /* Check if function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nMEMORY_ERROR(%d): %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nMEMORY_ERROR(%d): %s() failed - returned NULL pointer\n\n",
             id, funcname);
-    return(1); 
+    return(1);
   }
 
   return(0);
@@ -906,31 +901,31 @@ static int check_retval(void *returnvalue, const char *funcname, int opt, int id
  */
 
 /*
- * res: System residual function 
+ * res: System residual function
  *
  * To compute the residual function F, this routine calls:
  * rescomm, for needed communication, and then
- * reslocal, for computation of the residuals on this processor.      
+ * reslocal, for computation of the residuals on this processor.
  */
 
-static int res(realtype tt, 
-               N_Vector uv, N_Vector uvp, N_Vector rr, 
+static int res(realtype tt,
+               N_Vector uv, N_Vector uvp, N_Vector rr,
                void *user_data)
 {
   int retval;
   UserData data;
   sunindextype Nlocal;
-  
+
   data = (UserData) user_data;
-  
+
   Nlocal = data->n_local;
 
   /* Call rescomm to do inter-processor communication. */
   retval = rescomm(Nlocal, tt, uv, uvp, user_data);
-  
+
   /* Call reslocal to calculate the local portion of residual vector. */
   retval = reslocal(Nlocal, tt, uv, uvp, rr, user_data);
-  
+
   return(retval);
 }
 
@@ -941,10 +936,10 @@ static int res(realtype tt,
  * interior subgrid boundaries (ghost cell data).  It loads this data
  * into a work array cext (the local portion of c, extended).
  * The message-passing uses blocking sends, non-blocking receives,
- * and receive-waiting, in routines BRecvPost, BSend, BRecvWait.         
+ * and receive-waiting, in routines BRecvPost, BSend, BRecvWait.
  */
 
-static int rescomm(sunindextype Nlocal, realtype tt, 
+static int rescomm(sunindextype Nlocal, realtype tt,
                    N_Vector uv, N_Vector uvp,
                    void *user_data)
 {
@@ -954,31 +949,31 @@ static int rescomm(sunindextype Nlocal, realtype tt,
   int thispe, ixsub, jysub, nsmxsub, nsmysub;
   MPI_Comm comm;
   MPI_Request request[4];
-  
+
   data = (UserData) user_data;
-  cdata = N_VGetArrayPointer_Parallel(uv);
-  
+  cdata = N_VGetArrayPointer(uv);
+
   /* Get comm, thispe, subgrid indices, data sizes, extended array cext. */
-  
-  comm = data->comm;     
+
+  comm = data->comm;
   thispe = data->thispe;
 
-  ixsub = data->ixsub;   
+  ixsub = data->ixsub;
   jysub = data->jysub;
   gridext = data->gridext;
-  nsmxsub = data->nsmxsub; 
+  nsmxsub = data->nsmxsub;
   nsmysub = (data->ns)*(data->mysub);
-  
+
   /* Start receiving boundary data from neighboring PEs. */
-  BRecvPost(comm, request, thispe, ixsub, jysub, nsmxsub, nsmysub, 
+  BRecvPost(comm, request, thispe, ixsub, jysub, nsmxsub, nsmysub,
             gridext, buffer);
-  
+
   /* Send data from boundary of local grid to neighboring PEs. */
   BSend(comm, thispe, ixsub, jysub, nsmxsub, nsmysub, cdata);
-  
+
   /* Finish receiving boundary data from neighboring PEs. */
   BRecvWait(request, ixsub, jysub, nsmxsub, gridext, buffer);
-  
+
   return(0);
 }
 
@@ -987,7 +982,7 @@ static int rescomm(sunindextype Nlocal, realtype tt,
  * (1) buffer should be able to hold 2*NUM_SPECIES*MYSUB realtype entries,
  *     should be passed to both the BRecvPost and BRecvWait functions, and
  *     should not be manipulated between the two calls.
- * (2) request should have 4 entries, and is also passed in both calls. 
+ * (2) request should have 4 entries, and is also passed in both calls.
  */
 
 static void BRecvPost(MPI_Comm comm, MPI_Request request[], int my_pe,
@@ -1001,28 +996,28 @@ static void BRecvPost(MPI_Comm comm, MPI_Request request[], int my_pe,
 
   /* If jysub > 0, receive data for bottom x-line of cext. */
   if (jysub != 0)
-    MPI_Irecv(&cext[NUM_SPECIES], dsizex, PVEC_REAL_MPI_TYPE,
+    MPI_Irecv(&cext[NUM_SPECIES], dsizex, MPI_SUNREALTYPE,
               my_pe-NPEX, 0, comm, &request[0]);
-  
+
   /* If jysub < NPEY-1, receive data for top x-line of cext. */
   if (jysub != NPEY-1) {
     offsetce = NUM_SPECIES*(1 + (MYSUB+1)*(MXSUB+2));
-    MPI_Irecv(&cext[offsetce], dsizex, PVEC_REAL_MPI_TYPE,
+    MPI_Irecv(&cext[offsetce], dsizex, MPI_SUNREALTYPE,
               my_pe+NPEX, 0, comm, &request[1]);
   }
-  
+
   /* If ixsub > 0, receive data for left y-line of cext (via bufleft). */
   if (ixsub != 0) {
-    MPI_Irecv(&bufleft[0], dsizey, PVEC_REAL_MPI_TYPE,
+    MPI_Irecv(&bufleft[0], dsizey, MPI_SUNREALTYPE,
               my_pe-1, 0, comm, &request[2]);
   }
-  
+
   /* If ixsub < NPEX-1, receive data for right y-line of cext (via bufright). */
   if (ixsub != NPEX-1) {
-    MPI_Irecv(&bufright[0], dsizey, PVEC_REAL_MPI_TYPE,
+    MPI_Irecv(&bufright[0], dsizey, MPI_SUNREALTYPE,
               my_pe+1, 0, comm, &request[3]);
   }
-  
+
 }
 
 /*
@@ -1030,7 +1025,7 @@ static void BRecvPost(MPI_Comm comm, MPI_Request request[], int my_pe,
  * (1) buffer should be able to hold 2*NUM_SPECIES*MYSUB realtype entries,
  *     should be passed to both the BRecvPost and BRecvWait functions, and
  *     should not be manipulated between the two calls.
- * (2) request should have 4 entries, and is also passed in both calls.  
+ * (2) request should have 4 entries, and is also passed in both calls.
  */
 
 static void BRecvWait(MPI_Request request[], int ixsub, int jysub,
@@ -1040,13 +1035,13 @@ static void BRecvWait(MPI_Request request[], int ixsub, int jysub,
   int ly, dsizex2, offsetce, offsetbuf;
   realtype *bufleft = buffer, *bufright = buffer+NUM_SPECIES*MYSUB;
   MPI_Status status;
-  
+
   dsizex2 = dsizex + 2*NUM_SPECIES;
-  
+
   /* If jysub > 0, receive data for bottom x-line of cext. */
   if (jysub != 0)
     MPI_Wait(&request[0],&status);
-  
+
   /* If jysub < NPEY-1, receive data for top x-line of cext. */
   if (jysub != NPEY-1)
     MPI_Wait(&request[1],&status);
@@ -1063,7 +1058,7 @@ static void BRecvWait(MPI_Request request[], int ixsub, int jysub,
         cext[offsetce+i] = bufleft[offsetbuf+i];
     }
   }
-  
+
   /* If ixsub < NPEX-1, receive data for right y-line of cext (via bufright). */
   if (ixsub != NPEX-1) {
     MPI_Wait(&request[3],&status);
@@ -1081,9 +1076,9 @@ static void BRecvWait(MPI_Request request[], int ixsub, int jysub,
 /*
  * BSend: Send boundary data to neighboring PEs.
  * This routine sends components of uv from internal subgrid boundaries
- * to the appropriate neighbor PEs.                                      
+ * to the appropriate neighbor PEs.
  */
- 
+
 static void BSend(MPI_Comm comm, int my_pe, int ixsub, int jysub,
                   int dsizex, int dsizey, realtype cdata[])
 {
@@ -1094,13 +1089,13 @@ static void BSend(MPI_Comm comm, int my_pe, int ixsub, int jysub,
   /* If jysub > 0, send data from bottom x-line of uv. */
 
   if (jysub != 0)
-    MPI_Send(&cdata[0], dsizex, PVEC_REAL_MPI_TYPE, my_pe-NPEX, 0, comm);
+    MPI_Send(&cdata[0], dsizex, MPI_SUNREALTYPE, my_pe-NPEX, 0, comm);
 
   /* If jysub < NPEY-1, send data from top x-line of uv. */
 
   if (jysub != NPEY-1) {
     offsetc = (MYSUB-1)*dsizex;
-    MPI_Send(&cdata[offsetc], dsizex, PVEC_REAL_MPI_TYPE, my_pe+NPEX, 0, comm);
+    MPI_Send(&cdata[offsetc], dsizex, MPI_SUNREALTYPE, my_pe+NPEX, 0, comm);
   }
 
   /* If ixsub > 0, send data from left y-line of uv (via bufleft). */
@@ -1112,11 +1107,11 @@ static void BSend(MPI_Comm comm, int my_pe, int ixsub, int jysub,
       for (i = 0; i < NUM_SPECIES; i++)
         bufleft[offsetbuf+i] = cdata[offsetc+i];
     }
-    MPI_Send(&bufleft[0], dsizey, PVEC_REAL_MPI_TYPE, my_pe-1, 0, comm);   
+    MPI_Send(&bufleft[0], dsizey, MPI_SUNREALTYPE, my_pe-1, 0, comm);
   }
-  
+
   /* If ixsub < NPEX-1, send data from right y-line of uv (via bufright). */
-  
+
   if (ixsub != NPEX-1) {
     for (ly = 0; ly < MYSUB; ly++) {
       offsetbuf = ly*NUM_SPECIES;
@@ -1124,10 +1119,10 @@ static void BSend(MPI_Comm comm, int my_pe, int ixsub, int jysub,
       for (i = 0; i < NUM_SPECIES; i++)
         bufright[offsetbuf+i] = cdata[offsetc+i];
     }
-    MPI_Send(&bufright[0], dsizey, PVEC_REAL_MPI_TYPE, my_pe+1, 0, comm);   
+    MPI_Send(&bufright[0], dsizey, MPI_SUNREALTYPE, my_pe+1, 0, comm);
   }
 }
- 
+
 /* Define lines are for ease of readability in the following functions. */
 
 #define mxsub      (data->mxsub)
@@ -1157,28 +1152,28 @@ static void BSend(MPI_Comm comm, int my_pe, int ixsub, int jysub,
  * locations in cext.  Then the reaction and diffusion terms are
  * evaluated in terms of the cext array, and the residuals are formed.
  * The reaction terms are saved separately in the vector data->rates
- * for use by the preconditioner setup routine.                          
+ * for use by the preconditioner setup routine.
  */
 
-static int reslocal(sunindextype Nlocal, realtype tt, 
+static int reslocal(sunindextype Nlocal, realtype tt,
                     N_Vector uv, N_Vector uvp, N_Vector rr,
                     void *user_data)
 {
   realtype *uvdata, *uvpxy, *resxy, xx, yy, dcyli, dcyui, dcxli, dcxui, dx2, dy2;
-  realtype ixend, ixstart, jystart, jyend;
+  int ixend, ixstart, jystart, jyend;
   int ix, jy, is, i, locc, ylocce, locce;
   realtype rates[2];
   UserData data;
-  
+
   data = (UserData) user_data;
 
   /* Get data pointers, subgrid data, array sizes, work array cext. */
-  uvdata = N_VGetArrayPointer_Parallel(uv);
+  uvdata = N_VGetArrayPointer(uv);
 
   dx2 = dx * dx;
   dy2 = dy * dy;
 
-  
+
   /* Copy local segment of uv vector into the working extended array gridext. */
   locc = 0;
   locce = nsmxsub2 + NUM_SPECIES;
@@ -1190,11 +1185,11 @@ static int reslocal(sunindextype Nlocal, realtype tt,
 
   /* To facilitate homogeneous Neumann boundary conditions, when this is
      a boundary PE, copy data from the first interior mesh line of uv to gridext. */
-  
+
   /* If jysub = 0, copy x-line 2 of uv to gridext. */
   if (jysub == 0)
     { for (i = 0; i < nsmxsub; i++) gridext[NUM_SPECIES+i] = uvdata[nsmxsub+i]; }
-  
+
 
   /* If jysub = npey-1, copy x-line mysub-1 of uv to gridext. */
   if (jysub == npey-1) {
@@ -1202,7 +1197,7 @@ static int reslocal(sunindextype Nlocal, realtype tt,
     locce = (mysub+1)*nsmxsub2 + NUM_SPECIES;
     for (i = 0; i < nsmxsub; i++) gridext[locce+i] = uvdata[locc+i];
   }
-  
+
 
   /* If ixsub = 0, copy y-line 2 of uv to gridext. */
   if (ixsub == 0) {
@@ -1212,7 +1207,7 @@ static int reslocal(sunindextype Nlocal, realtype tt,
       for (i = 0; i < NUM_SPECIES; i++) gridext[locce+i] = uvdata[locc+i];
     }
   }
-  
+
 
   /* If ixsub = npex-1, copy y-line mxsub-1 of uv to gridext. */
   if (ixsub == npex-1) {
@@ -1222,17 +1217,17 @@ static int reslocal(sunindextype Nlocal, realtype tt,
       for (i = 0; i < NUM_SPECIES; i++) gridext[locce+i] = uvdata[locc+i];
     }
   }
-  
+
   /* Loop over all grid points, setting local array rates to right-hand sides.
      Then set rr values appropriately (ODE in the interior and DAE on the boundary)*/
-  ixend = ixstart = jystart = jyend = 0;  
+  ixend = ixstart = jystart = jyend = 0;
 
   if (jysub==0)      jystart = 1;
   if (jysub==npey-1) jyend   = 1;
   if (ixsub==0)      ixstart = 1;
   if (ixsub==npex-1) ixend   = 1;
 
-  for (jy = jystart; jy < mysub-jyend; jy++) { 
+  for (jy = jystart; jy < mysub-jyend; jy++) {
     ylocce = (jy+1)*nsmxsub2;
     yy     = (jy+jysub*mysub)*dy;
 
@@ -1240,18 +1235,18 @@ static int reslocal(sunindextype Nlocal, realtype tt,
       locce = ylocce + (ix+1)*NUM_SPECIES;
       xx = (ix + ixsub*mxsub)*dx;
 
-      ReactRates(xx, yy, &(gridext[locce]), rates, data);      
-      
-      resxy = IJ_Vptr(rr,ix,jy); 
-      uvpxy = IJ_Vptr(uvp,ix,jy); 
-      
+      ReactRates(xx, yy, &(gridext[locce]), rates, data);
+
+      resxy = IJ_Vptr(rr,ix,jy);
+      uvpxy = IJ_Vptr(uvp,ix,jy);
+
       for (is = 0; is < NUM_SPECIES; is++) {
         dcyli = gridext[locce+is]          - gridext[locce+is-nsmxsub2];
         dcyui = gridext[locce+is+nsmxsub2] - gridext[locce+is];
-        
+
         dcxli = gridext[locce+is]             - gridext[locce+is-NUM_SPECIES];
         dcxui = gridext[locce+is+NUM_SPECIES] - gridext[locce+is];
-        
+
         resxy[is] = uvpxy[is]-eps[is]*((dcxui-dcxli)/dx2+(dcyui-dcyli)/dy2)-rates[is];
       }
     }
@@ -1259,9 +1254,9 @@ static int reslocal(sunindextype Nlocal, realtype tt,
 
   if (jysub==0) {
     for (ix=0; ix<mxsub; ix++) {
-      
+
       locce = nsmxsub2 + NUM_SPECIES * (ix+1);
-      resxy = IJ_Vptr(rr,ix,0); 
+      resxy = IJ_Vptr(rr,ix,0);
 
       for (is=0; is<NUM_SPECIES; is++)
         resxy[is] = gridext[locce+is+nsmxsub2] - gridext[locce+is];
@@ -1271,21 +1266,21 @@ static int reslocal(sunindextype Nlocal, realtype tt,
   if (ixsub==npex-1) {
     for(jy=0; jy<mysub; jy++) {
       locce = (jy+1)*nsmxsub2 + nsmxsub2-NUM_SPECIES;
-      resxy = IJ_Vptr(rr,mxsub-1,jy); 
+      resxy = IJ_Vptr(rr,mxsub-1,jy);
 
       for (is=0; is<NUM_SPECIES; is++)
         resxy[is] = gridext[locce+is-NUM_SPECIES] - gridext[locce+is];
     }
   }
-  
+
   if (ixsub==0) {
     for (jy=0; jy<mysub; jy++) {
       locce = (jy+1)*nsmxsub2 + NUM_SPECIES;
-      resxy = IJ_Vptr(rr,0,jy); 
+      resxy = IJ_Vptr(rr,0,jy);
 
       for (is=0; is<NUM_SPECIES; is++)
         resxy[is] = gridext[locce+is-NUM_SPECIES] - gridext[locce+is];
-    }    
+    }
   }
 
   if (jysub==npey-1) {
@@ -1300,9 +1295,9 @@ static int reslocal(sunindextype Nlocal, realtype tt,
   return(0);
 }
 
-/* 
- * ReactRates: Evaluate reaction rates at a given spatial point.   
- * At a given (x,y), evaluate the array of ns reaction terms R.          
+/*
+ * ReactRates: Evaluate reaction rates at a given spatial point.
+ * At a given (x,y), evaluate the array of ns reaction terms R.
  */
 
 static void ReactRates(realtype xx, realtype yy, realtype *uvval, realtype *rates,
@@ -1328,7 +1323,7 @@ static int resB(realtype tt, N_Vector yy, N_Vector yp,
   UserData data;
   int retval;
   sunindextype Nlocal;
-  
+
   data = (UserData) user_dataB;
 
   Nlocal = data->n_local;
@@ -1344,7 +1339,7 @@ static int resB(realtype tt, N_Vector yy, N_Vector yp,
 
 
 
-static int resBlocal(sunindextype Nlocal, realtype tt, 
+static int resBlocal(sunindextype Nlocal, realtype tt,
                      N_Vector uv, N_Vector uvp,
                      N_Vector uvB, N_Vector uvpB, N_Vector rrB,
                      void *user_dataB)
@@ -1360,15 +1355,15 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
   data = (UserData) user_dataB;
 
   B = data->B;
- 
+
 
   /* Get data pointers, subgrid data, array sizes, work array cext. */
-  uvBdata = N_VGetArrayPointer_Parallel(uvB);
+  uvBdata = N_VGetArrayPointer(uvB);
 
   dx2 = dx * dx;
   dy2 = dy * dy;
 
-  
+
   /* Copy local segment of uv vector into the working extended array gridext. */
   locc = 0;
   locce = nsmxsub2 + NUM_SPECIES;
@@ -1381,7 +1376,7 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
   /* If jysub = 0, copy x-line 2 of uv to gridext. */
   if (jysub == 0)
     { for (i = 0; i < nsmxsub; i++) gridext[NUM_SPECIES+i] = uvBdata[nsmxsub+i]; }
-  
+
 
   /* If jysub = npey-1, copy x-line mysub-1 of uv to gridext. */
   if (jysub == npey-1) {
@@ -1389,7 +1384,7 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
     locce = (mysub+1)*nsmxsub2 + NUM_SPECIES;
     for (i = 0; i < nsmxsub; i++) gridext[locce+i] = uvBdata[locc+i];
   }
-  
+
 
   /* If ixsub = 0, copy y-line 2 of uv to gridext. */
   if (ixsub == 0) {
@@ -1399,7 +1394,7 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
       for (i = 0; i < NUM_SPECIES; i++) gridext[locce+i] = uvBdata[locc+i];
     }
   }
-  
+
 
   /* If ixsub = npex-1, copy y-line mxsub-1 of uv to gridext. */
   if (ixsub == npex-1) {
@@ -1409,22 +1404,22 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
       for (i = 0; i < NUM_SPECIES; i++) gridext[locce+i] = uvBdata[locc+i];
     }
   }
-  
+
   /* Loop over all grid points, setting local array rates to right-hand sides.
      Then set rr values appropriately (ODE in the interior and DAE on the boundary)*/
-  ixend = ixstart = jystart = jyend = 0;  
+  ixend = ixstart = jystart = jyend = 0;
 
   if (jysub==0)      jystart = 1;
   if (jysub==npey-1) jyend   = 1;
   if (ixsub==0)      ixstart = 1;
   if (ixsub==npex-1) ixend   = 1;
 
-  for (jy = jystart; jy < mysub-jyend; jy++) { 
+  for (jy = jystart; jy < mysub-jyend; jy++) {
     ylocce = (jy+1)*nsmxsub2;
 
     for (ix = ixstart; ix < mxsub-ixend; ix++) {
       locce = ylocce + (ix+1)*NUM_SPECIES;
-      
+
       uvxy  = IJ_Vptr(uv  ,ix,jy);
       uvBxy = IJ_Vptr(uvB ,ix,jy);
       uvpBxy= IJ_Vptr(uvpB,ix,jy);
@@ -1433,7 +1428,7 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
       for (is = 0; is < NUM_SPECIES; is++) {
         dcyli = gridext[locce+is]          - gridext[locce+is-nsmxsub2];
         dcyui = gridext[locce+is+nsmxsub2] - gridext[locce+is];
-        
+
         dcxli = gridext[locce+is]             - gridext[locce+is-NUM_SPECIES];
         dcxui = gridext[locce+is+NUM_SPECIES] - gridext[locce+is];
 
@@ -1441,16 +1436,16 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
       }
 
       /* now add rates */
-      rrBxy[0] += (uvBxy[0]-uvBxy[1])*(2*uvxy[0]*uvxy[1] - B) - uvBxy[0]; 
+      rrBxy[0] += (uvBxy[0]-uvBxy[1])*(2*uvxy[0]*uvxy[1] - B) - uvBxy[0];
       rrBxy[1] += uvxy[0]*uvxy[0]*(uvBxy[0]-uvBxy[1]);
     }
   }
 
   if (jysub==0) {
     for (ix=0; ix<mxsub; ix++) {
-      
+
       locce = nsmxsub2 + NUM_SPECIES * (ix+1);
-      rrBxy = IJ_Vptr(rrB,ix,0); 
+      rrBxy = IJ_Vptr(rrB,ix,0);
 
       for (is=0; is<NUM_SPECIES; is++)
         rrBxy[is] = gridext[locce+is+nsmxsub2] - gridext[locce+is];
@@ -1460,21 +1455,21 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
   if (ixsub==npex-1) {
     for(jy=0; jy<mysub; jy++) {
       locce = (jy+1)*nsmxsub2 + nsmxsub2-NUM_SPECIES;
-      rrBxy = IJ_Vptr(rrB,mxsub-1,jy); 
+      rrBxy = IJ_Vptr(rrB,mxsub-1,jy);
 
       for (is=0; is<NUM_SPECIES; is++)
         rrBxy[is] = gridext[locce+is-NUM_SPECIES] - gridext[locce+is];
     }
   }
-  
+
   if (ixsub==0) {
     for (jy=0; jy<mysub; jy++) {
       locce = (jy+1)*nsmxsub2 + NUM_SPECIES;
-      rrBxy = IJ_Vptr(rrB,0,jy); 
+      rrBxy = IJ_Vptr(rrB,0,jy);
 
       for (is=0; is<NUM_SPECIES; is++)
         rrBxy[is] = gridext[locce+is-NUM_SPECIES] - gridext[locce+is];
-    }    
+    }
   }
 
   if (jysub==npey-1) {
@@ -1489,4 +1484,3 @@ static int resBlocal(sunindextype Nlocal, realtype tt,
 
   return(0);
 }
-
