@@ -2,7 +2,7 @@
  * Programmer(s): Radu Serban and Cosmin Petra @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2020, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -13,28 +13,28 @@
  * -----------------------------------------------------------------
  * Adjoint sensitivity example problem
  *
- * This IVP is a stiff system of 6 non-linear DAEs of index 1. The 
- * problem originates from Akzo Nobel Central research in Arnhern, 
- * The Netherlands, and describes a chemical process in which 2 
+ * This IVP is a stiff system of 6 non-linear DAEs of index 1. The
+ * problem originates from Akzo Nobel Central research in Arnhern,
+ * The Netherlands, and describes a chemical process in which 2
  * species are mixed, while carbon dioxide is continuously added.
- * See http://pitagora.dm.uniba.it/~testset/report/chemakzo.pdf  
- * 
+ * See http://pitagora.dm.uniba.it/~testset/report/chemakzo.pdf
+ *
  * IDAS also computes the sensitivities with respect to initial
  * conditions of the following quantity:
  *   G = int_t0^t1 y1 dt
- * The sensitivity of G is the solution of the adjoint system at t0. 
+ * The sensitivity of G is the solution of the adjoint system at t0.
  * -----------------------------------------------------------------*/
- 
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
- 
+
 #include <idas/idas.h>                 /* prototypes for IDA fcts., consts.    */
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
 #include <sunmatrix/sunmatrix_dense.h> /* access to dense SUNMatrix            */
 #include <sunlinsol/sunlinsol_dense.h> /* access to dense SUNLinearSolver      */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
-#include <sundials/sundials_math.h>    /* defs. of SUNRabs, SUNRexp, etc.      */
+#include <sundials/sundials_math.h>    /* def. of SUNRpowerI                   */
 
 /* Accessor macros */
 #define Ith(v,i)    NV_Ith_S(v,i-1)       /* i-th vector component */
@@ -66,12 +66,12 @@ typedef struct {
 
 static int res(realtype t, N_Vector yy, N_Vector yd, N_Vector resval, void *userdata);
 
-static int resB(realtype tt, 
+static int resB(realtype tt,
                 N_Vector yy, N_Vector yp,
                 N_Vector yyB, N_Vector ypB, N_Vector rrB,
                 void *user_dataB);
 
-static int rhsQ(realtype t, N_Vector yy, N_Vector yp, 
+static int rhsQ(realtype t, N_Vector yy, N_Vector yp,
               N_Vector qdot, void *user_data);
 
 static void PrintOutput(realtype tfinal, N_Vector yB, N_Vector ypB);
@@ -142,7 +142,7 @@ int main()
   res(T0, yy, yp, rr, data);
   N_VScale(-ONE, rr, yp);
   N_VDestroy(rr);
-  
+
  /* Create and initialize q0 for quadratures. */
   q = N_VNew_Serial(1);
   if (check_retval((void *)q, "N_VNew_Serial", 0)) return(1);
@@ -186,7 +186,7 @@ int main()
 
   retval = IDASetQuadErrCon(mem, SUNTRUE);
   if (check_retval(&retval, "IDASetQuadErrCon", 1)) return(1);
- 
+
   /* Prepare ADJOINT. */
   retval = IDAAdjInit(mem, STEPS, IDA_HERMITE);
   if (check_retval(&retval, "IDAAdjInit", 1)) return(1);
@@ -208,7 +208,7 @@ int main()
   printf("G:          %24.16Lf \n",Ith(q,1));
 #else
   printf("G:          %24.16f \n",Ith(q,1));
-#endif  
+#endif
   printf("--------------------------------------------------------\n\n");
 
 
@@ -218,7 +218,7 @@ int main()
   yB = N_VNew_Serial(NEQ);
   if (check_retval((void *)yB, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, yB);
-  
+
   ypB = N_VNew_Serial(NEQ);
   if (check_retval((void *)ypB, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, ypB);
@@ -314,11 +314,11 @@ static int res(realtype t, N_Vector yy, N_Vector yd, N_Vector resval, void *user
   yd4 = Ith(yd,4);
   yd5 = Ith(yd,5);
 
-  r1 = k1 * SUNRpowerI(y1,4) * SUNRsqrt(y2);
+  r1 = k1 * SUNRpowerI(y1,4) * sqrt(y2);
   r2 = k2 * y3 * y4;
   r3 = k2/K * y1 * y5;
   r4 = k3 * y1 * y4 * y4;
-  r5 = k4 * y6 * y6 * SUNRsqrt(y2);
+  r5 = k4 * y6 * y6 * sqrt(y2);
   Fin = klA * ( pCO2/H - y2 );
 
   Ith(resval,1) = yd1 + TWO*r1 - r2 + r3 + r4;
@@ -331,13 +331,13 @@ static int res(realtype t, N_Vector yy, N_Vector yd, N_Vector resval, void *user
   return(0);
 }
 
-/* 
- * rhsQ routine. Computes quadrature(t,y). 
+/*
+ * rhsQ routine. Computes quadrature(t,y).
  */
- 
+
 static int rhsQ(realtype t, N_Vector yy, N_Vector yp, N_Vector qdot, void *user_data)
 {
-  Ith(qdot,1) = Ith(yy,1);  
+  Ith(qdot,1) = Ith(yy,1);
 
   return(0);
 }
@@ -346,10 +346,10 @@ static int rhsQ(realtype t, N_Vector yy, N_Vector yp, N_Vector qdot, void *user_
 #define FOUR      RCONST(4.0)
 #define EIGHT     RCONST(8.0)
 
-/* 
- * resB routine. Residual for adjoint system. 
+/*
+ * resB routine. Residual for adjoint system.
  */
-static int resB(realtype tt, 
+static int resB(realtype tt,
                 N_Vector yy, N_Vector yp,
                 N_Vector yyB, N_Vector ypB, N_Vector rrB,
                 void *user_dataB)
@@ -388,7 +388,7 @@ static int resB(realtype tt,
   yB4 = Ith(yyB,4);
   yB5 = Ith(yyB,5);
   yB6 = Ith(yyB,6);
-  
+
   ypB1 = Ith(ypB,1);
   ypB2 = Ith(ypB,2);
   ypB3 = Ith(ypB,3);
@@ -400,18 +400,18 @@ static int resB(realtype tt,
   k2overK = k2/K;
 
   tmp1 = k1* y1to3 * y2tohalf; tmp2 = k3*y4*y4;
-  Ith(rrB,1) = 1 +  ypB1 - (EIGHT*tmp1 + k2overK*y5 + tmp2)*yB1 
-    - (TWO*tmp1+tmp2)*yB2 + (FOUR*tmp1+k2overK*y5)*yB3 
+  Ith(rrB,1) = 1 +  ypB1 - (EIGHT*tmp1 + k2overK*y5 + tmp2)*yB1
+    - (TWO*tmp1+tmp2)*yB2 + (FOUR*tmp1+k2overK*y5)*yB3
     + k2overK*y5*(yB4-yB5) - TWO*tmp2*yB4 + Ks*y4*yB6;
 
   tmp1 = k1 * y1*y1to3 * (y2tohalf/y2); tmp2 = k4 * y6*y6 * (y2tohalf/y2);
-  Ith(rrB,2) = ypB2 - tmp1*yB1 - (QUARTER*tmp1 + QUARTER*tmp2 + klA)*yB2 
+  Ith(rrB,2) = ypB2 - tmp1*yB1 - (QUARTER*tmp1 + QUARTER*tmp2 + klA)*yB2
     + HALF*tmp1*yB3 + HALF*tmp2*yB5;
 
   Ith(rrB,3) = ypB3 + k2*y4*(yB1-yB3-yB4+yB5);
 
   tmp1 = k3*y1*y4; tmp2 = k2*y3;
-  Ith(rrB,4) = ypB4 + (tmp2-TWO*tmp1)*yB1 - TWO*tmp1*yB2 - tmp2*yB3 
+  Ith(rrB,4) = ypB4 + (tmp2-TWO*tmp1)*yB1 - TWO*tmp1*yB2 - tmp2*yB3
     - (tmp2+FOUR*tmp1)*yB4 + tmp2*yB5 + Ks*y1*yB6;
 
   Ith(rrB,5) = ypB5 - k2overK*y1*(yB1-yB3-yB4+yB5);
@@ -432,19 +432,19 @@ static void PrintOutput(realtype tfinal, N_Vector yB, N_Vector ypB)
   printf("dG/dy0: \t%12.4Le\n\t\t%12.4Le\n\t\t%12.4Le\n\t\t%12.4Le\n\t\t%12.4Le\n\t\t%12.4Le\n",
 #else
   printf("dG/dy0: \t%12.4e\n\t\t%12.4e\n\t\t%12.4e\n\t\t%12.4e\n\t\t%12.4e\n\t\t%12.4e\n",
-#endif         
+#endif
          Ith(yB,1), Ith(yB,2), Ith(yB,3), Ith(yB,4), Ith(yB,5), Ith(yB,6));
   printf("--------------------------------------------------------\n\n");
 }
 
-/* 
+/*
  * Check function return value.
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
  *    opt == 1 means SUNDIALS function returns an integer value so check if
  *             retval < 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
 static int check_retval(void *returnvalue, const char *funcname, int opt)
